// TT-Tachometer 12. for the v3.0 board with external clock feeding Timer1, works
// tests display and shuts off display if turntable not spinning.

// OLED libs and variables
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 32 // OLED display height, in pixels

#define OLED_RESET     -1 // Reset pin # (or -1 if sharing Arduino reset pin)
#define SCREEN_ADDRESS 0x3C ///< See datasheet for Address; 0x3D for 128x64, 0x3C for 128x32
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

#define LOGO_HEIGHT   16
#define LOGO_WIDTH    16

//Global Variables
#define TACH_PIN 2  
#define CLOCK_PIN 5
//set volatile for ISR variable
volatile float timePast = 0;
float previousTime = 0;
float rpm = 0;
float speed = 0;
volatile unsigned long num_overflows = 0;
unsigned long count = 0;

static const unsigned char PROGMEM logo_bmp[] =
{ 0b00000000, 0b11000000,
  0b00000001, 0b11000000,
  0b00000001, 0b11000000,
  0b00000011, 0b11100000,
  0b11110011, 0b11100000,
  0b11111110, 0b11111000,
  0b01111110, 0b11111111,
  0b00110011, 0b10011111,
  0b00011111, 0b11111100,
  0b00001101, 0b01110000,
  0b00011011, 0b10100000,
  0b00111111, 0b11100000,
  0b00111111, 0b11110000,
  0b01111100, 0b11110000,
  0b01110000, 0b01110000,
  0b00000000, 0b00110000 
};


void setup() {
  Serial.begin(9600);

    // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  if(!display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) {
    Serial.println(F("SSD1306 allocation failed"));
    for(;;); // Don't proceed, loop forever
  }

  // Show initial display buffer contents on the screen --
  // the library initializes this with an Adafruit splash screen.
  display.display();
  delay(1000); // Pause for 1 seconds
  display.setTextSize(2);             // Draw 2X-scale text
  display.setTextColor(SSD1306_WHITE);

  TCCR1B = 0b00000111; //external clock source
  TIMSK1 = 0b00000001; //enable Timer 1 overflow interrupt.
  TIMSK0 = 0b00000000; //disable Timer0 interrupts.
  TIMSK2 = 0b00000000; //disable Timer2 interrupts.
  TCCR1A = 0; //set WGM21 and WGM20 to 0
  pinMode(TACH_PIN, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(TACH_PIN), isr1, FALLING);
  pinMode(CLOCK_PIN, INPUT); 

  display.clearDisplay();
  display.setCursor(0, 0);
  display.println("No Spin");
  display.println("Detected");
  display.display();

} //end setup

// Interrupt Service Routine 1. Saves the time from the clock var.
void isr1(){
  timePast = get_count();
}

// Interrupt Service Routine 2. count the number of overflows.
// one overflow counts as 65536 clock pulses.
ISR(TIMER1_OVF_vect){
  num_overflows++;
}

volatile unsigned long get_count(){
  uint8_t SREG_Save = SREG; //Save the AVR Status Register
  noInterrupts();
  unsigned long tcnt1_Save = TCNT1; //save counter of Timer1
  boolean OVFFLAG = bitRead(TIFR1, 0); //Timer1 Overflow Flag
  if(OVFFLAG){
    tcnt1_Save = TCNT1; //re-read timer1 in case changed
    num_overflows++;
    TIFR1 |= 0b00000001; //reset Timer1 overflow flag
  }  
  count = ((num_overflows*65536) + tcnt1_Save);
  num_overflows = 0;
  TCNT1 = 0;
  SREG = SREG_Save; //enable interrupts as previously set
  return count;
}

void loop() {
  if (num_overflows > 5){  // turntable not turning

//   Serial.println("No Spin Detected");  // for debugging
   display.clearDisplay();
   display.setCursor(0, 0);
   display.println("No Spin");
   display.println("Detected");
   display.display();
  }
  else{ 
    if((timePast - previousTime) != 0){
      rpm = 6000000/timePast; //float rpm calc  
    }
    previousTime = timePast; //update previous time each pass, remove calc unless change
    drawttspeed(rpm);
   } 
} // end loop

// Displays given number on OLED screen truncated to 3 decimal points
void drawttspeed(float number) {
//  Serial.println(number);
  display.clearDisplay();
  display.setCursor(0, 0);
  display.print("RPM:");
  display.print(number, 3);
  display.display();
}//end drawttspeed()
